package service

import (
	"fmt"
	"iot_manager_service/app/device/dao"
	"iot_manager_service/app/device/model"
	"iot_manager_service/app/system/service"
	"iot_manager_service/util"
	"time"
)

// 中间件管理服务
var CaptureUintService = new(captureUintService)

type captureUintService struct{}

func (s *captureUintService) CaptureSubmit(userId int64, tenantId int, req *model.CaptureDetail) *util.Errors {
	device := req.CaptureUnit
	device.TenantId = tenantId
	device.UpdateUser = userId
	device.UpdateTime = time.Now()
	if gateway, err := GatewayService.GetOne(device.GatewayId); err == nil {
		device.GatewayName = gateway.GatewayName
		device.GatewaySn = gateway.GatewaySn
	}

	if device.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = userId

		if device.IsExistedBySN() {
			fmt.Printf("Create IsExistedBySN \n")
			return util.ParamsInvalidResponse(model.RepeatedPrompts, nil)
		}
		fmt.Printf("device = %+v \n", device)
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return util.FailResponse(err.Error(), nil)
		}
		service.OperationHisService.Save(userId, tenantId, util.OperationCreate, util.ModuleTypeDevice,
			util.DeviceTypeCaptureUnit, util.GetDeviceObject(device.ID, device.GatewayName), util.OperationSuccess)
		return util.SuccessResponse(util.Succeeded, nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return util.FailResponse(err.Error(), nil)
	}

	service.OperationHisService.Save(userId, tenantId, util.OperationUpdate, util.ModuleTypeDevice,
		util.DeviceTypeCaptureUnit, util.GetDeviceObject(device.ID, device.GatewayName), util.OperationSuccess)
	return util.SuccessResponse(util.Succeeded, nil)
}

func (s *captureUintService) GetCapture(id int) (*model.CaptureDetail, *util.Errors) {
	// 创建查询实例
	device := &dao.CaptureUnit{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	detail := &model.CaptureDetail{CaptureUnit: *device}
	endTime, state := util.GetDeviceState(device.CaptureSn)
	detail.NetworkState = state
	detail.EndLineTime = endTime

	return detail, nil
}

func (s *captureUintService) CaptureList(searchValue string, current, size int) ([]model.CaptureDetail, *util.Errors) {
	var details []model.CaptureDetail
	device := dao.CaptureUnit{}

	if searchValue != "" {
		device.CaptureSn = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)

	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}

	for _, d := range devices {
		endTime, state := util.GetDeviceState(d.CaptureSn)
		details = append(details, model.CaptureDetail{
			CaptureUnit:  d,
			EndLineTime:  endTime,
			NetworkState: state,
		})
	}

	return details, nil
}

func (s *captureUintService) CaptureGetList(tenantId int) ([]*dao.CaptureUnit, *util.Errors) {
	device := &dao.CaptureUnit{
		TenantId:  tenantId,
		IsDeleted: 0,
	}
	devices, err := device.GetAllDevices()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	for _, d := range devices {
		d.CaptureName = d.CaptureName + "(" + d.CaptureSn + ")"
	}
	return devices, nil
}

func (s *captureUintService) GetPoint(id int) (*dao.CheckPoint, *util.Errors) {
	// 创建查询实例
	device := &dao.CheckPoint{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}

	return device, nil
}

func (s *captureUintService) PointSubmit(userId int64, tenantId int, req *dao.CheckPoint) *util.Errors {
	device := req
	device.TenantId = tenantId
	device.UpdateUser = userId
	device.UpdateTime = time.Now()

	if device.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = userId

		if device.IsExistedBySN() {
			fmt.Printf("Create IsExistedBySN \n")
			return util.ParamsInvalidResponse(model.RepeatedPrompts, nil)
		}
		fmt.Printf("device = %+v \n", device)
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return util.FailResponse(err.Error(), nil)
		}
		service.OperationHisService.Save(userId, tenantId, util.OperationCreate, util.ModuleTypeDevice,
			util.DeviceTypePoint, util.GetDeviceObject(device.ID, device.PointName), util.OperationSuccess)
		return util.SuccessResponse(util.Succeeded, nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return util.FailResponse(err.Error(), nil)
	}

	service.OperationHisService.Save(userId, tenantId, util.OperationUpdate, util.ModuleTypeDevice,
		util.DeviceTypePoint, util.GetDeviceObject(device.ID, device.PointName), util.OperationSuccess)
	return util.SuccessResponse(util.Succeeded, nil)
}

func (s *captureUintService) PointList(searchValue string, current, size int) ([]dao.CheckPoint, *util.Errors) {
	device := dao.CheckPoint{}
	if searchValue != "" {
		device.PointSN = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	return devices, nil
}

func (s *captureUintService) PointGetList(tenantId int) ([]*dao.CheckPoint, *util.Errors) {
	device := &dao.CheckPoint{
		TenantId:  tenantId,
		IsDeleted: 0,
	}
	devices, err := device.GetAllDevices()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	for _, d := range devices {
		d.PointName = d.PointName + "(" + d.PointSN + ")"
	}
	return devices, nil
}

func (s *captureUintService) RemoveCapture(userId int64, tenantId int, id int) *util.Errors {
	// 创建查询实例
	device := &dao.CaptureUnit{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: userId,
		UpdateTime: time.Now(),
	}
	err := device.Delete()
	if err != nil {
		return util.FailResponse(err.Error(), nil)
	}
	service.OperationHisService.Save(userId, tenantId, util.OperationRemove, util.ModuleTypeDevice,
		util.DeviceTypeCaptureUnit, util.GetDeviceObject(device.ID, device.CaptureName), util.OperationSuccess)
	return nil
}

func (s *captureUintService) RemovePoint(userId int64, tenantId int, id int) *util.Errors {
	// 创建查询实例
	device := &dao.CheckPoint{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: userId,
		UpdateTime: time.Now(),
	}
	err := device.Delete()
	if err != nil {
		return util.FailResponse(err.Error(), nil)
	}
	service.OperationHisService.Save(userId, tenantId, util.OperationRemove, util.ModuleTypeDevice,
		util.DeviceTypePoint, util.GetDeviceObject(device.ID, device.PointName), util.OperationSuccess)
	return nil
}

func (s *captureUintService) GetCaptureByGateway(id int) []dao.CaptureUnit {
	// 创建查询实例
	device := &dao.CaptureUnit{
		GatewayId: id,
	}
	return device.GetDevicesByGateway()
}

func (s *captureUintService) GetByLampPole(id int) []dao.CaptureUnit {
	// 创建查询实例
	device := &dao.CaptureUnit{
		LampPoleId: id,
	}
	return device.GetDevicesByLampPole()
}
