package service

import (
	"fmt"
	"iot_manager_service/app/device/dao"
	"iot_manager_service/app/device/model"
	"iot_manager_service/app/system/service"
	"iot_manager_service/util"
	"time"
)

var GarbageService = new(garbageService)

type garbageService struct{}

func (s *garbageService) Get(id int) (*dao.Garbage, *util.Errors) {
	// 创建查询实例
	device := &dao.Garbage{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	return device, nil

}

func (s *garbageService) CreateOrUpdate(userId int64, tenantId int, req dao.Garbage) *util.Errors {
	// 创建查询实例
	device := req
	device.UpdateUser = userId
	device.TenantId = tenantId
	device.UpdateTime = time.Now()
	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = userId
		if device.IsExistedBySN() {
			fmt.Printf("Create GetDeviceID err \n")
			return util.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
		}
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return util.FailResponse(err.Error(), nil)
		}
		service.OperationHisService.Save(userId, tenantId, util.OperationCreate, util.ModuleTypeDevice,
			util.DeviceTypeGarbage, util.GetDeviceObject(device.ID, device.DeviceName), util.OperationSuccess)
		return util.SuccessResponse(util.Succeeded, nil)
	}
	if device.IsExistedByNameAndCode() {
		return util.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return util.FailResponse(err.Error(), nil)
	}
	service.OperationHisService.Save(userId, tenantId, util.OperationUpdate, util.ModuleTypeDevice,
		util.DeviceTypeGarbage, util.GetDeviceObject(device.ID, device.DeviceName), util.OperationSuccess)
	return util.SuccessResponse(util.Succeeded, nil)
}

func (s *garbageService) List(searchValue string, current, size int) ([]dao.Garbage, *util.Errors) {
	device := dao.Garbage{}
	if searchValue != "" {
		device.DeviceSN = searchValue
		device.DeviceName = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	return devices, nil
}

func (s *garbageService) Remove(userId int64, tenantId int, id int) *util.Errors {
	// 创建查询实例
	device := &dao.Garbage{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: userId,
		UpdateTime: time.Now(),
	}
	err := device.Delete()
	if err != nil {
		return util.FailResponse(err.Error(), nil)
	}
	service.OperationHisService.Save(userId, tenantId, util.OperationRemove, util.ModuleTypeDevice,
		util.DeviceTypeGarbage, util.GetDeviceObject(device.ID, device.DeviceName), util.OperationSuccess)
	return nil
}

func (s *garbageService) GetList(tenantId int) ([]*dao.Garbage, *util.Errors) {
	var devices []*dao.Garbage
	err := util.Redis.Get(getGarbageListRedisKey("000000")).Scan(&devices)
	if err == nil {
		return devices, nil
	}
	device := &dao.Garbage{
		TenantId:  tenantId,
		IsDeleted: 0,
	}
	devices, err = device.GetAllDevices()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	_ = util.Redis.Set(getAlarmListRedisKey(tenantId), devices, 0).Err()
	return devices, nil
}

func getGarbageListRedisKey(tenantId string) string {
	return fmt.Sprintf(model.GarbageList, tenantId)
}
