package service

import (
	"fmt"
	"iot_manager_service/app/device/dao"
	"iot_manager_service/app/device/model"
	"iot_manager_service/app/system/service"
	"iot_manager_service/util"
	"time"
)

var IpBroadcastService = new(ipBroadcastService)

type ipBroadcastService struct{}

func (s *ipBroadcastService) Get(id int) (*model.IpBroadcastDetail, *util.Errors) {
	// 创建查询实例
	device := &dao.IpBroadcast{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	endTime, state := util.GetDeviceState(device.CastSn)
	return &model.IpBroadcastDetail{
		IpBroadcast:  *device,
		RunState:     state,
		NetworkState: state,
		EndLineTime:  endTime,
	}, nil
}

func (s *ipBroadcastService) CreateOrUpdate(userId int64, tenantId int, req dao.IpBroadcast) *util.Errors {
	// 创建查询实例
	device := req
	device.TenantId = tenantId
	device.UpdateUser = userId
	device.UpdateTime = time.Now()
	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = userId
		if device.IsExistedBySN() {
			fmt.Printf("Create GetDeviceID err \n")
			return util.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
		}
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return util.FailResponse(err.Error(), nil)
		}
		service.OperationHisService.Save(userId, tenantId, util.OperationCreate, util.ModuleTypeDevice,
			util.DeviceTypeIPBroadcast, util.GetDeviceObject(device.ID, device.CastName), util.OperationSuccess)
		return util.SuccessResponse(util.Succeeded, nil)
	}
	if device.IsExistedByNameAndCode() {
		return util.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return util.FailResponse(err.Error(), nil)
	}

	service.OperationHisService.Save(userId, tenantId, util.OperationUpdate, util.ModuleTypeDevice,
		util.DeviceTypeIPBroadcast, util.GetDeviceObject(device.ID, device.CastName), util.OperationSuccess)
	return util.SuccessResponse(util.Succeeded, nil)
}

func (s *ipBroadcastService) List(searchValue string, current, size int) ([]model.IpBroadcastDetail, *util.Errors) {
	device := dao.IpBroadcast{}
	if searchValue != "" {
		device.CastSn = searchValue
		device.CastName = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	var details []model.IpBroadcastDetail
	for _, d := range devices {
		endTime, state := util.GetDeviceState(d.CastSn)
		details = append(details, model.IpBroadcastDetail{
			IpBroadcast:  d,
			RunState:     state,
			NetworkState: state,
			EndLineTime:  endTime,
		})
	}
	return details, nil
}

func (s *ipBroadcastService) Remove(userId int64, tenantId int, id int) *util.Errors {
	// 创建查询实例
	device := &dao.IpBroadcast{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: userId,
		UpdateTime: time.Now(),
	}
	err := device.Delete()
	if err != nil {
		return util.FailResponse(err.Error(), nil)
	}
	service.OperationHisService.Save(userId, tenantId, util.OperationRemove, util.ModuleTypeDevice,
		util.DeviceTypeIPBroadcast, util.GetDeviceObject(device.ID, device.CastName), util.OperationSuccess)
	return nil
}

func (s *ipBroadcastService) GetList(tenantId int) ([]dao.IpBroadcast, *util.Errors) {
	device := &dao.IpBroadcast{
		TenantId:  tenantId,
		IsDeleted: 0,
	}
	devices, err := device.GetAllDevices()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}

	return devices, nil
}

func (s *ipBroadcastService) GetByGateway(id int) []dao.IpBroadcast {
	// 创建查询实例
	device := &dao.IpBroadcast{
		GatewayId: id,
	}
	return device.GetDevicesByGateway()
}

func (s *ipBroadcastService) GetByLampPole(id int) []dao.IpBroadcast {
	// 创建查询实例
	device := &dao.IpBroadcast{
		LampPoleId: id,
	}
	return device.GetDevicesByLampPole()
}
