package service

import (
	"fmt"
	"iot_manager_service/app/device/dao"
	"iot_manager_service/app/system/service"
	"iot_manager_service/util"
	"time"
)

var OnDemandSensorService = new(onDemandSensorService)

type onDemandSensorService struct{}

func (s *onDemandSensorService) Get(id int) (*dao.OnDemandSensor, *util.Errors) {
	// 创建查询实例
	device := &dao.OnDemandSensor{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	return device, nil

}

func (s *onDemandSensorService) CreateOrUpdate(userId int64, tenantId int, req dao.OnDemandSensor) *util.Errors {
	// 创建查询实例
	device := req
	device.TenantId = tenantId
	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = userId
		if device.IsExistedBySN() {
			fmt.Printf("Create GetDeviceID err \n")
			return util.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
		}
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return util.FailResponse(err.Error(), nil)
		}
		service.OperationHisService.Save(userId, tenantId, util.OperationCreate, util.ModuleTypeDevice,
			util.DeviceTypeOnDemandSensor, util.GetDeviceObject(device.ID, device.Name), util.OperationSuccess)
		return util.SuccessResponse(util.Succeeded, nil)
	}
	device.UpdateUser = userId
	device.UpdateTime = time.Now()
	if device.IsExistedByNameAndCode() {
		return util.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return util.FailResponse(err.Error(), nil)
	}

	service.OperationHisService.Save(userId, tenantId, util.OperationUpdate, util.ModuleTypeDevice,
		util.DeviceTypeOnDemandSensor, util.GetDeviceObject(device.ID, device.Name), util.OperationSuccess)
	return util.SuccessResponse(util.Succeeded, nil)
}

func (s *onDemandSensorService) List(searchValue string, current, size int) ([]dao.OnDemandSensor, *util.Errors) {
	device := dao.OnDemandSensor{}
	if searchValue != "" {
		device.SN = searchValue
		device.Name = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	return devices, nil
}

func (s *onDemandSensorService) Remove(userId int64, tenantId int, id int) *util.Errors {
	// 创建查询实例
	device := &dao.OnDemandSensor{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: userId,
		UpdateTime: time.Now(),
	}
	err := device.Delete()
	if err != nil {
		return util.FailResponse(err.Error(), nil)
	}
	service.OperationHisService.Save(userId, tenantId, util.OperationRemove, util.ModuleTypeDevice,
		util.DeviceTypeOnDemandSensor, util.GetDeviceObject(device.ID, device.Name), util.OperationSuccess)
	return nil
}
